<?php
namespace Ems\portalGestor\Controller;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;

use Ems\portalGestor\Service\OcorrenciaService;

class OcorrenciaController extends Crud {
    /** @var OcorrenciaService */
    protected $ocorrenciaService;

    public function __construct(
        EntityManager $entityManager,
        OcorrenciaService $ocorrenciaService
    ) {
        $this->entityManager        = $entityManager;
        $this->ocorrenciaService   = $ocorrenciaService;
    }

    // public function getOcorrenciasPendentes(DTO\Request\Filter $request, DTO\Response $response) {
    //     try {
    //         $params = $request->getParameters();

    //         $nrorg = $params['NRORG'];
    //         $nrvinculom = $params['NRVINCULOM'];
    //         $nrperiodoapuracao = empty($params['NRPERIODOAPURACAO']) ? null : $params['NRPERIODOAPURACAO'];

    //         $totalOcorrenciasPendentes = $this->ocorrenciaService->getOcorrenciasPendentes(
    //             $nrorg,
    //             $nrvinculom,
    //             $nrperiodoapuracao
    //         );
    //         $response->addDataSet(new DataSet('data', $totalOcorrenciasPendentes));
    //     } catch (\Exception $e) {
    //         $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
    //     }
    // }

    public function getOcorrenciasPeriodo(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $params = $request->getParameters();

            $nrorg = $params['NRORG'];
            $nrvinculom = $params['NRVINCULOM'];
            $nrperiodoapuracao = empty($params['NRPERIODOAPURACAO']) ? null : $params['NRPERIODOAPURACAO'];

            $totalOcorrenciasPendentes = $this->ocorrenciaService->getOcorrenciasPeriodo(
                $nrorg,
                $nrvinculom,
                $nrperiodoapuracao
            );
            $response->addDataSet(new DataSet('data', $totalOcorrenciasPendentes));

        } catch (\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }

    public function setOcorrenciaFreq(Request\Row $request, DTO\Response $response){
        try {
            $row = $request->getRow();
    
            $success = $this->ocorrenciaService->setOcorrenciaFreq($row);

            $response->addDataSet(new DataSet('setOcorrenciaFreq', array('success' => $success)));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function removeOcorrenciaFreq(Request\Row $request, DTO\Response $response){
        try {
            $row = $request->getRow();
    
            $success = $this->ocorrenciaService->removeOcorrenciaFreq($row);

            $response->addDataSet(new DataSet('removeOcorrenciaFreq', array('success' => $success)));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }


    public function getOcorrenciasDia(DTO\Request\Filter $request, Response $response) {
        try {
            $params = $request->getParameters();
            $nrOrg             = $params['NRORG'];
            $nrVinculo         = $params['NRVINCULOM'];
            $dtinicioprogocor  = $params['DTINICIOPROGOCOR'];

            $dataSet = $this->ocorrenciaService->
                getOcorrenciasDia($nrOrg, $nrVinculo, $dtinicioprogocor);
            $response->addDataSet(new DataSet('data', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getTpOcorrencia(DTO\Request\Filter $request, Response $response) {
        try {
            $params = $request->getParameters();
            $nrOrg             = $params['NRORG'];

            $dataSet = $this->ocorrenciaService->
                getTpOcorrencia($nrOrg);
            $response->addDataSet(new DataSet('data', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getOcorrenciasPendentesPeriodoVinculosGestor(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $params = $request->getParameters();

            $nrorg = $params['NRORG'];
            $nrestruturam = $params['NRESTRUTURAM'];
            $nrperiodoapuracao = empty($params['NRPERIODOAPURACAO']) ? null : $params['NRPERIODOAPURACAO'];

            $totalOcorrenciasPendentes = $this->ocorrenciaService->getOcorrenciasPendentesPeriodoVinculosGestor(
                $nrorg,
                $nrestruturam,
                $nrperiodoapuracao
            );
            $response->addDataSet(new DataSet('getOcorrenciasPendentesPeriodoVinculosGestor', $totalOcorrenciasPendentes));

        } catch (\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }    
    }

    public function aprovarOcorrencia(
        DTO\Request\Row $request,
        DTO\Response $response
    ) {
        try {
            $params = $request->getRow();

            $nrorg = $params['NRORG'];
            $nrprogocorrencia = $params['NRPROGOCORRENCIA'];
            $cdoperador = $params['CDOPERADOR'];

            $success = $this->ocorrenciaService->aprovarOcorrencia(
                $nrorg,
                $nrprogocorrencia,
                $cdoperador
            );

            $response->addDataSet(new DataSet('aprovarOcorrencia', $success));

        } catch (\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }    
    }

    public function rejeitarOcorrencia(
        DTO\Request\Row $request,
        DTO\Response $response
    ) {
        try {
            $params = $request->getRow();

            $nrorg = $params['NRORG'];
            $nrprogocorrencia = $params['NRPROGOCORRENCIA'];
            $cdoperador = $params['CDOPERADOR'];

            $success = $this->ocorrenciaService->rejeitarOcorrencia(
                $nrorg,
                $nrprogocorrencia,
                $cdoperador
            );

            $response->addDataSet(new DataSet('rejeitarOcorrencia', $success));

        } catch (\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }

    public function getOcorrenciaFile(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $row = $request->getParameters();

            $nrOrg = (int) $row['NRORG'];
            $nrOcorrencia = (int) $row['NRPROGOCORRENCIA'];

            $file = $this->ocorrenciaService->getOcorrenciaFile(
                $nrOrg,
                $nrOcorrencia
            );

            $response->addDataSet(new DataSet('data', $file));

        } catch (\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }

    public function setOcorrenciasFreq(
        Request\Row $request,
        Response $response
    ){
        try {
            $row = $request->getRow();

            $success = $this->ocorrenciaService->
                setOcorrenciasFreq($row);

            $dataset = [
                'success' => $success
            ];

            $response->addDataSet(new DataSet('data', $dataset));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function removeOcorrenciasFreq(
        Request\Row $request,
        Response $response
    ) {
        try {
            $row = $request->getRow();

            $nrOrg      = (int)     $row['NRORG'];
            $nrVinculo  = (int)     $row['NRVINCULOM'];
            $dates      = (array)   $row['DATE'];
            $hrMarcacao = (string)  $row['HORAMARCACAO'];

            $success = $this->ocorrenciaService->removeOcorrenciasFreq(
                $nrOrg, $nrVinculo, $dates, $hrMarcacao
            );

            $dataset = [
                'success' => $success
            ];

            $response->addDataSet(new DataSet('data', $dataset));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function removeAlmocos(
        Request\Row $request,
        Response $response
    ) {
        try {
            $row = $request->getRow();

            $nrOrg      = (int)     $row['NRORG'];
            $nrVinculo  = (int)     $row['NRVINCULOM'];
            $dates      = (array)   $row['DATE'];

            $success = $this->ocorrenciaService->removeAlmocos(
                $nrOrg, $nrVinculo, $dates
            );

            $dataset = [
                'success' => $success
            ];

            $response->addDataSet(new DataSet('data', $dataset));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getEventosAtivos(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $row = $request->getParameters();
            $nrOrg = (int) $row['NRORG'];

            $eventos = $this->ocorrenciaService->getEventosAtivos(
                $nrOrg
            );

            $response->addDataSet(new DataSet('data', $eventos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getDiagnost(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $row = $request->getParameters();
            $nrOrg = (int) $row['NRORG'];

            $pessoaConselhoReg = $this->ocorrenciaService->getDiagnost(
                $nrOrg
            );

            $response->addDataSet(new DataSet('data', $pessoaConselhoReg));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getPessoaConselhoReg(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $row = $request->getParameters();
            $nrOrg = (int) $row['NRORG'];

            $pessoaConselhoReg = $this->ocorrenciaService->getPessoaConselhoReg(
                $nrOrg
            );

            $response->addDataSet(new DataSet('data', $pessoaConselhoReg));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }
}